/*
 * arch/microblaze/kernel/procfs.c -- Introspection functions for /proc filesystem
 *
 *  Copyright (C) 2003       John Williams <jwilliams@itee.uq.edu.au>
 *
 *  based heavily on v850 code that was
 *
 *  Copyright (C) 2001,2002  NEC Corporation
 *  Copyright (C) 2001,2002  Miles Bader <miles@gnu.org>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 * Written by Miles Bader <miles@gnu.org>
 */

#include "mach.h"

/* Simple test to see if this is an auto-config architecture - 
   if so, make use of it */
#ifdef CONFIG_XILINX_MICROBLAZE0_INSTANCE
/* Helper function to output boolean cpuinfo values */
int cpuinfo_print_boolean(struct seq_file *m, char *param_name, 
				int value)
{					
	return seq_printf(m, "%-25s : %s\n", param_name, 
			value ? "yes" : "no");
}

/* Helper function to dump details of a cache configuration 
   (yes/no/size) */
int cpuinfo_print_cache(struct seq_file *m, char *cache_name, 
				int has_cache, int cache_size)
{
	if(has_cache)
		return seq_printf (m, "%-25s : %ukB\n", cache_name,
			cache_size >> 10);
	else
		return seq_printf (m, "%-25s : no\n", cache_name);
}
 
static int cpuinfo_print (struct seq_file *m, void *v)
{
	extern unsigned long loops_per_jiffy;
	int count=0;
	count = seq_printf (m,
		"CPU-Family                : MicroBlaze\n"
		"FPGA-Arch                 : %s\n"
		"CPU-Ver                   : %s\n"
		"CPU-MHz                   : %lu.%02lu\n"
		"BogoMips                  : %u.%02u\n",
		CONFIG_XILINX_MICROBLAZE0_FAMILY,
		CONFIG_XILINX_MICROBLAZE0_HW_VER,
		CONFIG_XILINX_CPU_CLOCK_FREQ/1000000,
		CONFIG_XILINX_CPU_CLOCK_FREQ % 1000000,
		loops_per_jiffy/(500000/HZ),
		(loops_per_jiffy/(5000/HZ)) % 100);

	/* Following sequence of boolean config params are tested
	   with ifdef first, since these params don't exist in MPD / 
	   auto-config for all CPU versions.  If the param is defined,
	   then the cpuinfo_print_boolean helper function is used
	   to actually output the string.  This keeps us backward
	   compatible with older CPU versions. */

#ifdef CONFIG_XILINX_MICROBLAZE0_USE_DIV
	count += cpuinfo_print_boolean(m,"HW-Div",
			CONFIG_XILINX_MICROBLAZE0_USE_DIV);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_USE_HW_MUL
	count += cpuinfo_print_boolean(m,"HW-Mul",
			CONFIG_XILINX_MICROBLAZE0_USE_HW_MUL);
#endif


#ifdef CONFIG_XILINX_MICROBLAZE0_USE_BARREL
	count += cpuinfo_print_boolean(m,"HW-Shift",
			CONFIG_XILINX_MICROBLAZE0_USE_BARREL);
#endif
	
#ifdef CONFIG_XILINX_MICROBLAZE0_USE_FPU
	count += cpuinfo_print_boolean(m,"FPU",
			CONFIG_XILINX_MICROBLAZE0_USE_FPU);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_USE_MSR_INSTR
	count += cpuinfo_print_boolean(m,"MSR instr",
			CONFIG_XILINX_MICROBLAZE0_USE_MSR_INSTR);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_USE_PCMP_INSTR
	count += cpuinfo_print_boolean(m,"PCMP instr",
			CONFIG_XILINX_MICROBLAZE0_USE_PCMP_INSTR);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_UNALIGNED_EXCEPTIONS
	count += cpuinfo_print_boolean(m,"Unaligned exception",
			CONFIG_XILINX_MICROBLAZE0_UNALIGNED_EXCEPTIONS);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_ILL_OPCODE_EXCEPTION
	count += cpuinfo_print_boolean(m,"Ill. opcode exception",
			CONFIG_XILINX_MICROBLAZE0_ILL_OPCODE_EXCEPTION);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_IOPB_BUS_EXCEPTION
	count += cpuinfo_print_boolean(m,"IOPB bus exception",
			CONFIG_XILINX_MICROBLAZE0_IOPB_BUS_EXCEPTION);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_DOPB_BUS_EXCEPTION
	count += cpuinfo_print_boolean(m,"DOPB bus exception",
			CONFIG_XILINX_MICROBLAZE0_DOPB_BUS_EXCEPTION);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_DIV_ZERO_EXCEPTION
	count += cpuinfo_print_boolean(m,"Div 0 exception",
			CONFIG_XILINX_MICROBLAZE0_DIV_ZERO_EXCEPTION);
#endif

#ifdef CONFIG_XILINX_MICROBLAZE0_FPU_EXCEPTION
	count += cpuinfo_print_boolean(m,"FPU exception",
			CONFIG_XILINX_MICROBLAZE0_FPU_EXCEPTION);
#endif

	count += cpuinfo_print_cache(m,"Icache",
			CONFIG_XILINX_MICROBLAZE0_USE_ICACHE,
			CONFIG_XILINX_MICROBLAZE0_CACHE_BYTE_SIZE);

	count += cpuinfo_print_cache(m,"Dcache",
			CONFIG_XILINX_MICROBLAZE0_USE_DCACHE,
			CONFIG_XILINX_MICROBLAZE0_DCACHE_BYTE_SIZE);

	count += cpuinfo_print_boolean(m,"HW-Debug",
		CONFIG_XILINX_MICROBLAZE0_DEBUG_ENABLED);

	return count;
}
#else /* !CONFIG_XILINX_MICROBLAZE0_INSTANCE */
static int cpuinfo_print (struct seq_file *m, void *v)
{
	extern unsigned long loops_per_jiffy;
	return seq_printf (m,
			   "CPU-Family               : MicroBlaze\n"
			   "CPU-Arch                 : %s\n"
			   "CPU-Model                : %s\n"
			   "CPU-MHz                  : %lu.%02lu\n"
			   "BogoMips                 : %u.%02u\n",
			   CPU_ARCH,
			   CPU_MODEL,
			   CONFIG_XILINX_CPU_CLOCK_FREQ/1000000,
			   CONFIG_XILINX_CPU_CLOCK_FREQ % 1000000,
			   loops_per_jiffy/(500000/HZ),
			   (loops_per_jiffy/(5000/HZ)) % 100);
}
#endif /* CONFIG_XILINX_MICROBLAZE0_INSTANCE */

static void *cpuinfo_start (struct seq_file *m, loff_t *pos)
{
	return *pos < NR_CPUS ? ((void *) 0x12345678) : NULL;
}

static void *cpuinfo_next (struct seq_file *m, void *v, loff_t *pos)
{
	++*pos;
	return cpuinfo_start (m, pos);
}

static void cpuinfo_stop (struct seq_file *m, void *v)
{
}

struct seq_operations cpuinfo_op = {
	start:	cpuinfo_start,
	next:	cpuinfo_next,
	stop:	cpuinfo_stop,
	show:	cpuinfo_print
};
