#
# common.mak -- Generic build instructions for Generic microblaze-auto PetaLinux system
#

include $(LINUX_CONFIG)
include $(CONFIG_CONFIG)
include $(ARCH_CONFIG)

LINUX	 = $(ROOTDIR)/$(LINUXDIR)/linux 
LINUXBIN = $(IMAGEDIR)/linux.bin
ROMFSIMG = $(IMAGEDIR)/romfs.img
JFFS2IMG = $(IMAGEDIR)/jffs2.img
IMAGE    = $(IMAGEDIR)/image.bin
ELFIMAGE = $(IMAGEDIR)/image.elf
CKSUM	 = $(ROOTDIR)/tools/cksum
SREC_IMAGE= $(IMAGEDIR)/image.srec

COMMON=$(VEND)/PetaLogix/common

# Do some sanity checks on the selected root filesystem type
ifdef CONFIG_SYSTEM_ROOTFS_CRAMFS
    USE_CRAMFS=y
    ifndef CONFIG_CRAMFS
        $(error CramFS was selected as the root filesystem type, but your kernel is not configured for CramFS support.)
    endif
endif
ifdef CONFIG_SYSTEM_ROOTFS_ROMFS
    ifndef CONFIG_FS_ROMFS
        $(error RomFS was selected as the root filesystem type, but your kernel is not configured for RomFS support.)
    endif
endif
ifdef CONFIG_SYSTEM_ROOTFS_JFFS2
    $(error Sorry, JFFS2 root filesystems are not yet supported.)
endif

DIRS = 
ROMFS_DIRS = bin dev dev/flash etc home lib mnt proc usr var

ifdef CONFIG_SYSTEM_ROOTFS_JFFS2
    ROMFS_DIRS += tmp var/run var/log var/tmp /var/lock var/empty
	FSIMG = $(JFFS2IMG)
else
	FSIMG = $(ROMFSIMG)
endif

ifdef CONFIG_USER_FLATFSD_FLATFSD
	ETCDIR = /etc/default
else
	ETCDIR = /etc
endif

# Only do this automatic processing, if platforms vendor Makefile
# hasn't already specified FLASH_DEVICES
ifndef FLASH_DEVICES
ifdef CONFIG_SYSTEM_MTDPARTS
# Crack open the MTDPARTS config variable to create named partition devices 
# in /dev/flash/XXXX
	FLASH_DEVICES = $(shell echo $(CONFIG_SYSTEM_MTDPARTS) | \
				awk -f $(COMMON)/parse_mtdparts.awk)
else
	include $(COMMON)/flash_devices.mak
endif
endif


DEVICES =								      \
	tty,c,5,0      console,c,5,1   mem,c,1,1     kmem,c,1,2    null,c,1,3 \
									      \
	ttyS0,c,4,64   ttyS1,c,4,65    ttyS2,c,4,66  ttyS3,c,4,67	      \
	rtc,c,10,135   nvram,c,10,144					      \
									      \
	zero,c,1,5     random,c,1,8    urandom,c,1,9			      \
									      \
        fb0,c,29,0                                                            \
                                                                              \
	ptyp0,c,2,0    ptyp1,c,2,1     ptyp2,c,2,2   ptyp3,c,2,3	      \
	ptyp4,c,2,4    ptyp5,c,2,5     ptyp6,c,2,6   ptyp7,c,2,7	      \
	ptyp8,c,2,8    ptyp9,c,2,9     ptypa,c,2,10  ptypb,c,2,11	      \
	ptypc,c,2,12   ptypd,c,2,13    ptype,c,2,14  ptypf,c,2,15	      \
									      \
	ttyp0,c,3,0    ttyp1,c,3,1     ttyp2,c,3,2   ttyp3,c,3,3	      \
	ttyp4,c,3,4    ttyp5,c,3,5     ttyp6,c,3,6   ttyp7,c,3,7	      \
	ttyp8,c,3,8    ttyp9,c,3,9     ttypa,c,3,10  ttypb,c,3,11	      \
	ttypc,c,3,12   ttypd,c,3,13    ttype,c,3,14  ttypf,c,3,15	   

DEVICES +=   \
	mtd0,c,90,0	mtdr0,c,90,1	mtdblock0,b,31,0	\
	mtd1,c,90,2	mtdr1,c,90,3	mtdblock1,b,31,1	\
	mtd2,c,90,4	mtdr2,c,90,5	mtdblock2,b,31,2	\
	mtd3,c,90,6	mtdr3,c,90,7	mtdblock3,b,31,3	\
	mtd4,c,90,8 	mtdr4,c,90,9	mtdblock4,b,31,4	\
	mtd5,c,90,10 	mtdr5,c,90,11	mtdblock5,b,31,5	\
	mtd6,c,90,12 	mtdr6,c,90,13	mtdblock6,b,31,6	\
	mtd7,c,90,14 	mtdr7,c,90,15	mtdblock7,b,31,7	\
	mtd8,c,90,16 	mtdr8,c,90,17	mtdblock8,b,31,8	\
	mtd9,c,90,18 	mtdr9,c,90,19	mtdblock9,b,31,9	\
	mtd10,c,90,20 	mtdr10,c,90,21	mtdblock10,b,31,10  \
	mtd11,c,90,22 	mtdr11,c,90,23	mtdblock11,b,31,11	\
	mtd12,c,90,24 	mtdr12,c,90,25	mtdblock12,b,31,12	\
	mtd13,c,90,26 	mtdr13,c,90,27	mtdblock13,b,31,13	\
	mtd14,c,90,26 	mtdr14,c,90,29	mtdblock14,b,31,14

ifdef CONFIG_MICROBLAZE_FSLFIFO
DEVICES += 	\
	fslfifo0,c,10,192 fslfifo1,c,10,193 fslfifo2,c,10,194		\
	fslfifo3,c,10,195 fslfifo4,c,10,196 fslfifo5,c,10,197		\
	fslfifo6,c,10,198 fslfifo7,c,10,199
endif

ifdef CONFIG_XILINX_SYSACE                                                  
DEVICES +=    \
	xsysacea,b,254,0        xsysacea1,b,254,1       xsysacea2,b,254,2  \
	xsysacea3,b,254,3       xsysacea4,b,254,4       xsysacea5,b,254,5  \
	xsysacea6,b,254,6       xsysacea7,b,254,7       xsysacea8,b,254,8  \
	xsysacea9,b,254,9       xsysacea10,b,254,10     xsysacea11,b,254,11 \
	xsysacea12,b,254,12     xsysacea12,b,254,13     xsysacea14,b,254,14 \
	xsysacea15,b,254,15 
endif 
      
# sd? dev nodes if SCSI is enabled
#ifdef CONFIG_SCSI
DEVICES +=	sda,b,8,0	\
	sda1,b,8,1	sda2,b,8,2	sda3,b,8,3	sda4,b,8,4	\
	sda5,b,8,5	sda6,b,8,6	sda7,b,8,7	sda8,b,8,8	\
	sda9,b,8,9	sda10,b,8,10	sda11,b,8,11	sda12,b,8,12	\
	sda13,b,8,13	sda14,b,8,14	sda15,b,8,15			\
	sdb,b,8,16							\
	sdb1,b,8,17	sdb2,b,8,18	sdb3,b,8,19	sdb4,b,8,20	\
	sdb5,b,8,21	sdb6,b,8,22	sdb7,b,8,23	sdb8,b,8,24	\
	sdb9,b,8,25	sdb10,b,8,26	sdb11,b,8,27	sdb12,b,8,28	\
	sdb13,b,8,29	sdb14,b,8,30	sdb15,b,8,31
#endif

ifdef CONFIG_XILINX_GPIO
DEVICES +=      \
        gpio,c,10,185
endif

ifdef CONFIG_INPUT_EVDEV
DEVICES +=      \
event0,c,13,64  event1,c,13,65  event2,c,13,66  event3,c,13,67 \
event4,c,13,68  event5,c,13,69  event6,c,13,70  event7,c,13,71 \
event8,c,13,72  event9,c,13,73  event10,c,13,74  event11,c,13,75 \
event12,c,13,76  event13,c,13,77  event14,c,13,78  event15,c,13,79
endif

#ifdef CONFIG_VIDEO_DEV
DEVICES +=	\
	video0,c,81,0
#endif

I2C_MAJOR = 89

ifndef CONFIG_SYSTEM_ROOT_PASSWD
CONFIG_SYSTEM_ROOT_PASSWD := "root"
endif
CRYPT_PASSWD       := $(shell perl $(ROOTDIR)/tools/crypt.pl $(CONFIG_SYSTEM_ROOT_PASSWD))

all::
	dirs=$(DIRS) ; \
	for i in $$dirs ; do  $(MAKE) -C $$i || exit $? ; done

clean::
	-dirs=$(DIRS) ; \
	for i in $$dirs; do [ ! -d $$i ] || $(MAKE) -C $$i clean; done
	-rm -f mkcramfs

romfs::
	[ -d $(ROMFSDIR)/$$i ] || mkdir -p $(ROMFSDIR)
	for i in $(ROMFS_DIRS); do \
		[ -d $(ROMFSDIR)/$$i ] || mkdir -p $(ROMFSDIR)/$$i; \
	done
	for i in $(DIRS); do $(MAKE) -C $$i romfs || exit $? ; done

	mkdir -p $(ROMFSDIR)/etc/init.d
	mkdir -p $(ROMFSDIR)/etc/rc.d

	# Symbolic link for fstab into configurable area
	$(ROMFSINST) -e CONFIG_USER_FLATFSD_FLATFSD \
		-s /etc/config/fstab /etc/fstab

	$(ROMFSINST) -e CONFIG_USER_FLATFSD_FLATFSD \
		-s /etc/config/passwd /etc/passwd

	$(ROMFSINST) -s /bin /sbin

ifndef CONFIG_SYSTEM_ROOTFS_JFFS2
	for i in $(DEVICES); do \
		touch $(ROMFSDIR)/dev/@$$i; \
	done
	# these permissions are needed for openpty and family to work
	# on non-ptmx ptys
	chmod 622 $(ROMFSDIR)/dev/@[pt]ty[pqrsPQRS][0-9a-f],*
	for i in $(FLASH_DEVICES); do \
		touch $(ROMFSDIR)/dev/flash/@$$i; \
	done
	$(ROMFSINST) -s /var/tmp /tmp

	# make I2C nodes if necessary 
ifdef CONFIG_I2C
	for i in 0 1 2 3 4 5 6 7; do \
		touch $(ROMFSDIR)/dev/@i2c-$$i,c,$(I2C_MAJOR),$$i; \
	done
endif #IIC

else
	$(ROMFSINST) $(COMMON)/etc/rc/checkroot /etc/init.d/checkroot
	$(ROMFSINST) -s /etc/init.d/checkroot /etc/rc.d/S01checkroot
endif

ifdef CONFIG_USER_FLATFSD_FLATFSD
	mkdir -p $(ROMFSDIR)/etc/config
	mkdir -p $(ROMFSDIR)$(ETCDIR)
	$(ROMFSINST) $(COMMON)/etc/rc/flatfsd /etc/init.d/flatfsd
	$(ROMFSINST) -s /etc/init.d/flatfsd /etc/rc.d/S02flatfsd
endif

	$(ROMFSINST) -a "root:${CRYPT_PASSWD}:0:0::/:/bin/sh" $(ETCDIR)/passwd

ifdef CONFIG_USER_INIT_INIT
	$(ROMFSINST) -e CONFIG_USER_AGETTY_AGETTY \
			-a "ttyS0:vt100:/bin/agetty ttyS0 115200" /etc/inittab
	$(ROMFSINST) -e CONFIG_USER_FLATFSD_FLATFSD \
			-a "flt:unknown:/bin/flatfsd" /etc/inittab
	$(ROMFSINST) -e CONFIG_USER_INETD_INETD \
			-a "inet:unknown:/bin/inetd" /etc/inittab
	$(ROMFSINST) -e CONFIG_USER_FLATFSD_FLATFSD \
			$(COMMON)/etc/rc/start $(ETCDIR)/start
	$(ROMFSINST) -e CONFIG_USER_FLATFSD_FLATFSD \
			$(COMMON)/etc/inittab.config $(ETCDIR)/inittab
	$(ROMFSINST) $(COMMON)/etc/rc/sysinit /etc/rc
endif

ifdef CONFIG_USER_BUSYBOX_INIT
	$(ROMFSINST) $(COMMON)/etc/rc/sysinit /etc/rc.sysinit
	$(ROMFSINST) $(COMMON)/etc/rc/reboot /etc/rc.reboot
ifdef CONFIG_USER_AGETTY_AGETTY 
	$(ROMFSINST) $(COMMON)/etc/inittab.agetty /etc/inittab
else
	$(ROMFSINST) $(COMMON)/etc/inittab.sh /etc/inittab
endif
endif
	$(ROMFSINST) -e CONFIG_USER_BUSYBOX_HOSTNAME \
			$(COMMON)/etc/rc/hostname /etc/init.d/hostname
	$(ROMFSINST) -e CONFIG_USER_SYSUTILS_HOSTNAME  \
			$(COMMON)/etc/rc/hostname /etc/init.d/hostname
	$(ROMFSINST) -e CONFIG_USER_NET_TOOLS_HOSTNAME \
			$(COMMON)/etc/rc/hostname /etc/init.d/hostname
	[ -e $(ROMFSDIR)/etc/init.d/hostname ] && \
		$(ROMFSINST) -a "/bin/hostname ${CONFIG_SYSTEM_HOSTNAME}" /etc/init.d/hostname
	[ -e $(ROMFSDIR)/etc/init.d/hostname ] && \
		$(ROMFSINST) -s /etc/init.d/hostname /etc/rc.d/S30hostname


############################################

ifdef CONFIG_NET

ifdef CONFIG_SYSTEM_USE_DHCP
    ifndef CONFIG_USER_DHCPCD_NEW_DHCPCD
        $(error A dynamic IP address was requested, but dhcpcd-new is not enabled in your vendor/user settings.)
    endif
endif

# setup /etc/init.d/network and/or /etc/resolv.conf (and hosts file)
ifdef CONFIG_SYSTEM_USE_DHCP
	$(ROMFSINST) $(COMMON)/etc/rc/dhcpcd-new /etc/init.d/network
else
	[ -f $(COMMON)/etc/resolv.conf ] && $(ROMFSINST) $(COMMON)/etc/resolv.conf $(ETCDIR)/resolv.conf
	[ -f $(COMMON)/etc/hosts ] && $(ROMFSINST) $(COMMON)/etc/hosts $(ETCDIR)/hosts
	$(ROMFSINST) -e CONFIG_USER_FLATFSD_FLATFSD \
		-s /etc/config/hosts /etc/hosts
endif

# this is the special case, overwrite /etc/init.d/network
ifdef CONFIG_SYSTEM_USE_DHCP
  ifdef CONFIG_MTD
    ifndef CONFIG_USER_FLATFSD_FLATFSD
	$(ROMFSINST)  $(COMMON)/etc/rc/dhcpcd-new.no-resolv.conf \
					/etc/init.d/network
    endif
  else
	mkdir -p $(ROMFSDIR)/etc/dhcpc
  endif
else
	$(ROMFSINST) $(COMMON)/etc/rc/ifconfig.lo /etc/init.d/network
	$(ROMFSINST) -e CONFIG_NETDEVICES $(COMMON)/etc/rc/ifconfig.eth /etc/init.d/network
	$(ROMFSINST) -e CONFIG_XILINX_ENET $(COMMON)/etc/rc/ifconfig.eth /etc/init.d/network
	$(ROMFSINST) -e CONFIG_NETDEVICES -a "ifconfig eth0 ${CONFIG_SYSTEM_IP_ADDRESS}" /etc/init.d/network
	$(ROMFSINST) -e CONFIG_XILINX_ENET -a "ifconfig eth0 ${CONFIG_SYSTEM_IP_ADDRESS}" /etc/init.d/network
endif	

ifdef CONFIG_USER_PORTMAP_PORTMAP
	$(ROMFSINST) -a " " /etc/init.d/network
	$(ROMFSINST) -a "echo \"Starting portmap:\"" /etc/init.d/network
	$(ROMFSINST) -a "/bin/portmap &" /etc/init.d/network
endif

	$(ROMFSINST) -s /etc/init.d/network /etc/rc.d/S40network

# Web servers
#
# thttpd - runs as a daemon, so install into init.d
# Web servers
#
# thttpd - runs as a daemon
# Web servers
#
# thttpd - runs as a daemon
ifdef CONFIG_USER_THTTPD_THTTPD
	$(ROMFSINST) $(COMMON)/etc/rc/thttpd /etc/init.d/thttpd
	$(ROMFSINST) -s /etc/init.d/thttpd /etc/rc.d/S90thttpd
	$(ROMFSINST) $(COMMON)/http-content /home/httpd
endif
#
# fnord web server - runs as as an inetd service
#
	$(ROMFSINST) -e CONFIG_USER_FNORD_HTTPD $(COMMON)/http-content /home/httpd

	$(ROMFSINST) ../../Generic/romfs/etc/services /etc/services
endif
	# Ensure init.d scripts are executable
	chmod +x $(ROMFSDIR)/etc/init.d/*

$(LINUXBIN): $(ROOTDIR)/$(LINUXDIR)/linux
	[ -d $(IMAGEDIR) ] || mkdir -p $(IMAGEDIR)
	$(CROSS)objcopy -O binary $(LINUX) $@

$(ROMFSIMG): $(LINUXBIN)
ifdef CONFIG_BINFMT_ZFLAT
	$(CROSS)flthdr -z $(ROMFSDIR)/bin/*
endif
ifdef USE_CRAMFS
	./mkcramfs -z -r $(ROMFSDIR) $(ROMFSIMG)
else
	genromfs -v -V "ROMdisk" -x *~ -f $(ROMFSIMG) -d $(ROMFSDIR)
endif
	cat $(LINUXBIN) $(ROMFSIMG) > $(IMAGE)

$(JFFS2IMG):
# current mkfs.jffs2 in uclinux dist doesn't work.
# assume we have good working mkfs.jffs2 installed on host system
	mkfs.jffs2 --big-endian --pad=1966080 --squash --eraseblock=65536 \
                   --devtable jffs2_devtable.txt --root=$(ROMFSDIR) --output=$(JFFS2IMG)
	cp $(LINUXBIN) $(IMAGE)
	dd if=$(JFFS2IMG) of=$(IMAGE) bs=1024 seek=1472

mkcramfs: $(ROOTDIR)/user/cramfs/mkcramfs.c
	$(HOSTCC) -o $@ -I$(ROOTDIR)/$(LINUXDIR)/include $< -lz

ifneq ($(BUILD_UBOOT),)
# Add U-Boot post image processing
.PHONY: mkubimage
mkubimage: 
	# Copy u-boot image to our image directory
	@cp $(UBOOTDIR)/$(UBOOTIMG).* $(IMAGEDIR)
	# Copy u-boot non-stubbed srec for FS-Boot bootstrap
	@cp $(UBOOTDIR)/u-boot.srec $(IMAGEDIR)

	# Check if mkimage tool is compiled
	[ -f $(MKIMAGE) ] || (echo "Error: U-boot mkimage tool not found." && exit -1)

    # Run mkimage to build u-boot bootable kernel image
    # FIXME - Compression hardcoded to none
	$(MKIMAGE) -A $(ARCH) -O linux -T kernel -C none \
	-a $(ERAM_START) -e $(ERAM_START) -n "PetaLinux Kernel 2.4"  \
	-d $(KERNEL_IMG) $(UB_KERNEL_IMG)

.PHONY: mkubscript
mkubscript: 
	# Run mkimage to build u-boot autoscr script
	$(MKIMAGE) -A $(ARCH) -O linux -T script -C none \
	-a 0 -e 0 -n "PetaLinux Autoscr Script"  \
	-d $(UBOOT_SCRIPT) $(UBOOT_SCRIPT_IMG)
endif

.PHONY: image $(FSIMG)
ifneq ($(BUILD_UBOOT),)
image: mkcramfs $(LINUXBIN) $(FSIMG) $(ELF_IMAGE) $(SREC_IMAGE) mkubimage mkubscript
else
image: mkcramfs $(LINUXBIN) $(FSIMG) $(ELF_IMAGE) $(SREC_IMAGE)
endif
ifdef CONFIG_USER_NETFLASH_VERSION
	echo -ne "\000" >> $(IMAGE)
	echo -n $(VERSIONPKG) >> $(IMAGE)
	echo -ne "\000" >> $(IMAGE)
	echo -n $(CONFIG_VENDOR) >> $(IMAGE)
	echo -ne "\000" >> $(IMAGE)
	echo -n $(CONFIG_PRODUCT) >> $(IMAGE)
endif
ifdef CONFIG_USER_NETFLASH_HMACMD5
	$(ROOTDIR)/user/netflash/hmacmd5 $(CONFIG_USER_NETFLASH_HMACMD5_KEY) < $(IMAGE) >> $(IMAGE)
else
	$(CKSUM) -b -o 2 $(IMAGE) >> $(IMAGE)
endif
	# Copy image files into /tftpboot, if requested
	if [ "$(CONFIG_SYSTEM_COPY_TO_TFTPBOOT)" == 'y' ] ; then \
		cp $(IMAGEDIR)/* $(CONFIG_SYSTEM_TFTPBOOT_DIR); \
	fi


$(SREC_IMAGE): $(ELFIMAGE)
	$(CROSS)objcopy -O srec $(ELFIMAGE) $(SREC_IMAGE)

$(ELFIMAGE): $(ROMFSIMG) $(LINUX)
	BSS=`$(CROSS)objdump --headers $(LINUX) | \
		grep "\.bss"` ; \
	ADDR=`set -- $${BSS} ; echo 0x$${5}` ; \
	$(CROSS)objcopy --add-section=.romfs=$(ROMFSIMG) \
		--adjust-section-vma=.romfs=$${ADDR} --no-adjust-warnings \
		--set-section-flags=.romfs=alloc,load,data   \
		$(LINUX) $(ELFIMAGE)

