/**
 * \file xtea.h
 *
 *  Copyright (C) 2009  Paul Bakker <polarssl_maintainer at polarssl dot org>
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of PolarSSL or XySSL nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef TROPICSSL_XTEA_H
#define TROPICSSL_XTEA_H

#define XTEA_ENCRYPT     1
#define XTEA_DECRYPT     0

/**
 * \brief          XTEA context structure
 */
typedef struct {
	unsigned long k[4];	/*!< key */
} xtea_context;

#ifdef __cplusplus
extern "C" {
#endif

	/**
	 * \brief          XTEA key schedule
	 *
	 * \param ctx      XTEA context to be initialized
	 * \param key      the secret key
	 */
	void xtea_setup(xtea_context * ctx, unsigned char key[16]);

	/**
	 * \brief          XTEA cipher function
	 *
	 * \param ctx      XTEA context
	 * \param mode     XTEA_ENCRYPT or XTEA_DECRYPT
	 * \param input    8-byte input block
	 * \param output   8-byte output block
	 */
	void xtea_crypt(xtea_context * ctx,
			int mode,
			unsigned char input[8], unsigned char output[8]);

	/*
	 * \brief          Checkup routine
	 *
	 * \return         0 if successful, or 1 if the test failed
	 */
	int xtea_self_test(int verbose);

#ifdef __cplusplus
}
#endif
#endif				/* xtea.h */
