/******************************************************************************
 *
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 * AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
 * SOLUTIONS FOR XILINX DEVICES. BY PROVIDING THIS DESIGN, CODE,
 * OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
 * APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
 * THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 * AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 * FOR YOUR IMPLEMENTATION. XILINX EXPRESSLY DISCLAIMS ANY
 * WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 * IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 * REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 * INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * (C) Copyright 2007-2008 Michal Simek
 * Michal SIMEK <monstr@monstr.eu>
 *
 * (c) Copyright 2003 Xilinx Inc.
 * All rights reserved.
 *
 ******************************************************************************/

#include <config.h>
#include <common.h>
#include <net.h>
#include <asm/io.h>

#include <asm/asm.h>

#undef DEBUG
#define DEBUG

typedef struct {
	u32 regbaseaddress;	/* Base address of registers */
	u32 databaseaddress;	/* Base address of data for FIFOs */
} xpacketfifov100b;

typedef struct {
	u32 baseaddress;	/* Base address (of IPIF) */
	u32 isstarted;		/* Device is currently started 0-no, 1-yes */
	xpacketfifov100b recvfifo;	/* FIFO used to receive frames */
	xpacketfifov100b sendfifo;	/* FIFO used to send frames */
} xemac;

#define PRI_MII 0x1f
#define SEC_MII 0x0f

#define XIIF_V123B_IISR_OFFSET	0x220UL /* IP interrupt status register 0x20*/
#define XIIF_V123B_RESET_MASK		0xAUL
#define XIIF_V123B_RESETR_OFFSET	0x100UL /* reset register */

/* This constant is used with the Reset Register */
#define XPF_RESET_FIFO_MASK		0x0000000A
#define XPF_COUNT_STATUS_REG_OFFSET	4UL

/* These constants are used with the Occupancy/Vacancy Count Register. This
 * register also contains FIFO status */
#define XPF_COUNT_MASK			0x0000FFFF
#define XPF_DEADLOCK_MASK		0x20000000

/* Offset of the MAC registers from the IPIF base address */
#define XEM_REG_OFFSET		0x1100UL

#define DEMAC_TXFIFO_DEPTH 1024 // This is the domain crossing FIFO
#define DEMAC_RXFIFO_DEPTH 1024 // This is the Rx Pkt FIFO

#define DEMAC_USER_SLAVE_SPACE_OFFSET (0x00000000)

/* DEMAC Write Regs */
#define DEMAC_TXFLEN (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x000000004)
#define DEMAC_MII_ADDR (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x00000008)
#define DEMAC_MII_TXDATA  (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x0000000C)
#define DEMAC_MII_CMD  (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x00000010)

/* DEMAC Read Regs */
#define DEMAC_RXFLEN (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x00000000)
#define DEMAC_TXDATA_CNT  (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x00000004)
#define DEMAC_RXLEN_EMPTY  (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x00000008)
#define DEMAC_MII_RXDATA  (DEMAC_USER_SLAVE_SPACE_OFFSET + 0x0000000C)

/**
 * IPIF Reset/Mir Space Register Offsets
 * -- RST : software reset register
 * -- MIR : module identification register
 */
#define DEMAC_IPIF_RST_SPACE_OFFSET (0x00000100)
#define DEMAC_RST_OFFSET (DEMAC_IPIF_RST_SPACE_OFFSET + 0x00000000)

/**
 * IPIF Read Packet FIFO Register/Data Space Offsets
 * -- RDFIFO_RST   : read packet fifo reset register
 * -- RDFIFO_MIR   : read packet fifo module identification register
 * -- RDFIFO_SR    : read packet fifo status register
 * -- RDFIFO_DATA  : read packet fifo data
 */
#define DEMAC_IPIF_RDFIFO_REG_SPACE_OFFSET (0x00000300)
#define DEMAC_RDFIFO_RST_OFFSET (DEMAC_IPIF_RDFIFO_REG_SPACE_OFFSET + 0x00000000)
#define DEMAC_RDFIFO_MIR_OFFSET (DEMAC_IPIF_RDFIFO_REG_SPACE_OFFSET + 0x00000000)
#define DEMAC_RDFIFO_SR_OFFSET (DEMAC_IPIF_RDFIFO_REG_SPACE_OFFSET + 0x00000004)
#define DEMAC_IPIF_RDFIFO_DATA_SPACE_OFFSET (0x00000400)
#define DEMAC_RDFIFO_DATA_OFFSET (DEMAC_IPIF_RDFIFO_DATA_SPACE_OFFSET + 0x00000000)

/**
 * IPIF Read Packet FIFO Masks
 * -- RDFIFO_EMPTY_MASK : read packet fifo empty condition
 * -- RDFIFO_AE_MASK    : read packet fifo almost empty condition
 * -- RDFIFO_DL_MASK    : read packet fifo deadlock condition
 * -- RDFIFO_SCL_MASK   : read packet fifo occupancy scaling enabled
 * -- RDFIFO_WIDTH_MASK : read packet fifo encoded data port width
 * -- RDFIFO_OCC_MASK   : read packet fifo occupancy
 */
#define RDFIFO_EMPTY_MASK (0x80000000UL)
#define RDFIFO_AE_MASK (0x40000000UL)
#define RDFIFO_DL_MASK (0x20000000UL)
#define RDFIFO_SCL_MASK (0x10000000UL)
#define RDFIFO_WIDTH_MASK (0x0E000000UL)
#define RDFIFO_OCC_MASK (0x01FFFFFFUL)

/**
 * IPIF Write Packet FIFO Register/Data Space Offsets
 * -- WRFIFO_RST   : write packet fifo reset register
 * -- WRFIFO_MIR   : write packet fifo module identification register
 * -- WRFIFO_SR    : write packet fifo status register
 * -- WRFIFO_DATA  : write packet fifo data
 */
#define DEMAC_IPIF_WRFIFO_REG_SPACE_OFFSET (0x00000500)
#define DEMAC_WRFIFO_RST_OFFSET (DEMAC_IPIF_WRFIFO_REG_SPACE_OFFSET + 0x00000000)
#define DEMAC_WRFIFO_MIR_OFFSET (DEMAC_IPIF_WRFIFO_REG_SPACE_OFFSET + 0x00000000)
#define DEMAC_WRFIFO_SR_OFFSET (DEMAC_IPIF_WRFIFO_REG_SPACE_OFFSET + 0x00000004)
#define DEMAC_IPIF_WRFIFO_DATA_SPACE_OFFSET (0x00000600)
#define DEMAC_WRFIFO_DATA_OFFSET (DEMAC_IPIF_WRFIFO_DATA_SPACE_OFFSET + 0x00000000)

/**
 * IPIF Write Packet FIFO Masks
 * -- WRFIFO_FULL_MASK  : write packet fifo full condition
 * -- WRFIFO_AF_MASK    : write packet fifo almost full condition
 * -- WRFIFO_DL_MASK    : write packet fifo deadlock condition
 * -- WRFIFO_SCL_MASK   : write packet fifo vacancy scaling enabled
 * -- WRFIFO_WIDTH_MASK : write packet fifo encoded data port width
 * -- WRFIFO_VAC_MASK   : write packet fifo vacancy
 */
#define WRFIFO_FULL_MASK (0x80000000UL)
#define WRFIFO_AF_MASK (0x40000000UL)
#define WRFIFO_DL_MASK (0x20000000UL)
#define WRFIFO_SCL_MASK (0x10000000UL)
#define WRFIFO_WIDTH_MASK (0x0E000000UL)
#define WRFIFO_VAC_MASK (0x01FFFFFFUL)

/**
 * IPIF Interrupt Controller Space Offsets
 * -- INTR_DISR  : device (ipif) interrupt status register
 * -- INTR_DIPR  : device (ipif) interrupt pending register
 * -- INTR_DIER  : device (ipif) interrupt enable register
 * -- INTR_DIIR  : device (ipif) interrupt id (priority encoder) register
 * -- INTR_DGIER : device (ipif) global interrupt enable register
 * -- INTR_ISR   : ip (user logic) interrupt status register
 * -- INTR_IER   : ip (user logic) interrupt enable register
 */
#define DEMAC_IPIF_INTR_SPACE_OFFSET (0x00000200)
#define DEMAC_INTR_DISR_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x00000000)
#define DEMAC_INTR_DIPR_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x00000004)
#define DEMAC_INTR_DIER_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x00000008)
#define DEMAC_INTR_DIIR_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x00000018)
#define DEMAC_INTR_DGIER_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x0000001C)
#define DEMAC_INTR_ISR_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x00000020)
#define DEMAC_INTR_IER_OFFSET (DEMAC_IPIF_INTR_SPACE_OFFSET + 0x00000028)
/**
 * IPIF Interrupt Controller Masks
 * -- INTR_TERR_MASK : transaction error
 * -- INTR_DPTO_MASK : data phase time-out
 * -- INTR_IPIR_MASK : ip interrupt requeset
 * -- INTR_DMA0_MASK : dma channel 0 interrupt request
 * -- INTR_DMA1_MASK : dma channel 1 interrupt request
 * -- INTR_RFDL_MASK : read packet fifo deadlock interrupt request
 * -- INTR_WFDL_MASK : write packet fifo deadlock interrupt request
 * -- INTR_IID_MASK  : interrupt id
 * -- INTR_GIE_MASK  : global interrupt enable
 * -- INTR_NOPEND    : the DIPR has no pending interrupts
 */
#define INTR_TERR_MASK (0x00000001UL)
#define INTR_DPTO_MASK (0x00000002UL)
#define INTR_IPIR_MASK (0x00000004UL)
#define INTR_DMA0_MASK (0x00000008UL)
#define INTR_DMA1_MASK (0x00000010UL)
#define INTR_RFDL_MASK (0x00000020UL)
#define INTR_WFDL_MASK (0x00000040UL)
#define INTR_IID_MASK (0x000000FFUL)
#define INTR_GIE_MASK (0x80000000UL)
#define INTR_NOPEND (0x80)

/* DEMAC ISR Masks */
#define DEMAC_INTR_XMIT_DONE_MASK            (0x00000001UL)
#define DEMAC_INTR_RECV_DONE_MASK            (0x00000002UL)
#define DEMAC_INTR_AUTO_NEG_MASK             (0x00000004UL) // N/A
#define DEMAC_INTR_RECV_REJECT_MASK          (0x00000008UL) // N/A
#define DEMAC_INTR_XMIT_SFIFO_MASK           (0x00000010UL) // N/A
#define DEMAC_INTR_RECV_LFIFO_EMPTY_MASK     (0x00000020UL) 
#define DEMAC_INTR_XMIT_LFIFO_FULL_MASK      (0x00000040UL) 
#define DEMAC_INTR_RECV_LFIFO_OVER_MASK      (0x00000080UL) // N/A
#define DEMAC_INTR_RECV_LFIFO_UNDER_MASK     (0x00000100UL) // N/A
#define DEMAC_INTR_XMIT_SFIFO_OVER_MASK      (0x00000200UL) // N/A
#define DEMAC_INTR_XMIT_SFIFO_UNDER_MASK     (0x00000400UL) // N/A
#define DEMAC_INTR_XMIT_LFIFO_OVER_MASK      (0x00000800UL) // N/A
#define DEMAC_INTR_XMIT_LFIFO_UNDER_MASK     (0x00001000UL) // N/A
#define DEMAC_INTR_RECV_PFIFO_ABORT_MASK     (0x00002000UL) // N/A
#define DEMAC_INTR_MII_PEND_MASK             (0x00004000UL) 
#define DEMAC_INTR_MII_DONE_MASK             (0x00008000UL)


/* MII_CMD */
#define DEMAC_MII_CMD_WRITE 0x1
#define DEMAC_MII_CMD_READ 0x2
#define DEMAC_MII_CMD_SETDIV 0x4
#define DEMAC_MII_CMD_EMAC_READ 0x8
#define DEMAC_MII_CMD_EMAC_WRITE 0x10
#define DEMAC_MII_CMD_MDIODIV_SHIFT 8
#define DEMAC_MII_CMD_MDIODIV_MASK 0xFF00
#define DEMAC_MII_CMD_CMD_MASK 0x000000FF

/* MII_ADDR */
#define DEMAC_MII_ADDR_PHYAD_MASK 0x001F
#define DEMAC_MII_ADDR_REGAD_MASK 0x1F00
#define DEMAC_MII_ADDR_REGAD_SHIFT 8
#define DEMAC_MII_ADDR_ETH0 1
#define DEMAC_MII_ADDR_ETH1 2
#define DEMAC_MII_ADDR_SPEED 1
#define DEMAC_MII_ADDR_MAC 2
#define DEMAC_MII_SPEED1000 1
#define DEMAC_MII_SPEED10_100 0

/* DanteRegs */
#define DANTE_APCP0_REG_BASE XPAR_DANTE_REG_0_AR0_BASEADDR
#define DANTE_APCP_CPU_MAC_H	0x40
#define DANTE_APCP_CPU_MAC_L	0x44
#define DANTE_APCP_CPU_APE_H	0x80
#define DANTE_APCP_CPU_APE_L	0x84
#define DANTE_APCP_CPU_IP	0x48
#define DANTE_APCP_APE_IP	0x88
#define DANTE_APCP_APE_UDP	0x8c

#define ENET_MAX_MTU		PKTSIZE
#define ENET_ADDR_LENGTH	6

static unsigned int etherrxbuff[PKTSIZE_ALIGN/4]; /* Receive buffer */

static u8 emacaddr[ENET_ADDR_LENGTH] = { 0x00, 0x1d, 0xc1, 0xfe, 0xfe, 0xfe };

static xemac emac;

void eth_phy_read(uint8_t phyaddr, uint8_t regaddr, uint16_t *data)
{
    uint32_t reg;
    volatile uint32_t intrstatus;
    uint32_t miidata;

    debug("Phy 0x%x read reg 0x%x\n", phyaddr, regaddr);
    
    reg = ((regaddr << DEMAC_MII_ADDR_REGAD_SHIFT) & DEMAC_MII_ADDR_REGAD_MASK) |
		     (phyaddr & DEMAC_MII_ADDR_PHYAD_MASK);
    out_be32 (emac.baseaddress + DEMAC_MII_ADDR, reg);
    out_be32 (emac.baseaddress + DEMAC_MII_CMD, DEMAC_MII_CMD_READ);

    //Spin MII done
    do {
	    intrstatus = in_be32 ((emac.baseaddress) +
					    DEMAC_INTR_ISR_OFFSET);
    }
    while ((intrstatus & DEMAC_INTR_MII_DONE_MASK) == 0);
    
    miidata = in_be32((emac.baseaddress) + DEMAC_MII_RXDATA);
    *data = (uint16_t)(miidata & 0xffff);

    //Clear ISR
    out_be32 (emac.baseaddress + DEMAC_INTR_ISR_OFFSET,
		intrstatus & (DEMAC_INTR_MII_DONE_MASK | DEMAC_INTR_MII_PEND_MASK));
}

void eth_halt(void)
{
	debug ("eth_halt\n");
}

int eth_init(bd_t * bis)
{
	uint32_t mac_h, mac_l;
	uint8_t *macptr_l,*macptr_h;
	uint16_t reg, spd;
	uint32_t speed;

	debug ("EMAC Initialization Started\n\r");
#if 0 //Always init to keep IP in sync
	if (emac.isstarted) {
		puts("Emac is started\n");
		return 0;
	}
#endif 
	memset (&emac, 0, sizeof (xemac));

	emac.baseaddress = DANTE_TEMAC_BASEADDR;

	/* Setting up FIFOs */
	emac.recvfifo.regbaseaddress = emac.baseaddress +
					DEMAC_IPIF_RDFIFO_REG_SPACE_OFFSET;
	emac.recvfifo.databaseaddress = emac.baseaddress +
					DEMAC_IPIF_RDFIFO_DATA_SPACE_OFFSET;
	out_be32 (emac.recvfifo.regbaseaddress, XPF_RESET_FIFO_MASK);

	emac.sendfifo.regbaseaddress = emac.baseaddress +
					DEMAC_IPIF_WRFIFO_REG_SPACE_OFFSET;
	emac.sendfifo.databaseaddress = emac.baseaddress +
					DEMAC_IPIF_WRFIFO_DATA_SPACE_OFFSET;
	out_be32 (emac.sendfifo.regbaseaddress, XPF_RESET_FIFO_MASK);

	/* Reset the entire IPIF */
	out_be32 (emac.baseaddress + XIIF_V123B_RESETR_OFFSET,
					XIIF_V123B_RESET_MASK);


	eth_phy_read(PRI_MII, 28, &reg);
	spd = reg&0x18;
	if(spd == 0x10){
	    printf("Gigabit Link found\n");
	    speed = DEMAC_MII_SPEED1000;
	}
	else if(spd == 0x8){
	    printf("100meg Link found\n");
	    speed = DEMAC_MII_SPEED10_100;
	}
	else
	    printf("No link found!!\n");

	/*Set PLL to correct speed, this must toggle back and forth */
	out_be32 (emac.baseaddress + DEMAC_MII_ADDR, ((DEMAC_MII_ADDR_SPEED << DEMAC_MII_ADDR_REGAD_SHIFT) & DEMAC_MII_ADDR_REGAD_MASK) | (0x01 & DEMAC_MII_ADDR_PHYAD_MASK));
	out_be32 (emac.baseaddress + DEMAC_MII_TXDATA,
		    (speed == DEMAC_MII_SPEED1000) ? DEMAC_MII_SPEED10_100:DEMAC_MII_SPEED1000);
	out_be32 (emac.baseaddress + DEMAC_MII_CMD, DEMAC_MII_CMD_EMAC_WRITE);
	out_be32 (emac.baseaddress + DEMAC_MII_ADDR, ((DEMAC_MII_ADDR_SPEED << DEMAC_MII_ADDR_REGAD_SHIFT) & DEMAC_MII_ADDR_REGAD_MASK) | (0x01 & DEMAC_MII_ADDR_PHYAD_MASK));
	out_be32 (emac.baseaddress + DEMAC_MII_TXDATA,
		    (speed == DEMAC_MII_SPEED1000) ? DEMAC_MII_SPEED1000:DEMAC_MII_SPEED10_100);
	out_be32 (emac.baseaddress + DEMAC_MII_CMD, DEMAC_MII_CMD_EMAC_WRITE);

	udelay(500000); //PLL Must be stable before we write to Dante reg's
	
	if (!getenv("ethaddr")) {
		printf("ethaddr env problems?");
		memcpy(bis->bi_enetaddr, emacaddr, ENET_ADDR_LENGTH);
	}
	
	mac_h = (bis->bi_enetaddr[0]<<8) | (bis->bi_enetaddr[1]);
	mac_l = (bis->bi_enetaddr[2]<<24) | (bis->bi_enetaddr[3]<<16) | (bis->bi_enetaddr[4]<<8) | (bis->bi_enetaddr[5]);
	macptr_h = &mac_h;
	macptr_l = &mac_l;
	printf("Found MAC: %x:%x:%x:%x:%x:%x\n", macptr_h[2],macptr_h[3],
			macptr_l[0],macptr_l[1],macptr_l[2],macptr_l[3]);

	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_MAC_H), mac_h);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_MAC_L), mac_l);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_IP), bis->bi_ip_addr);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_APE_H), mac_h);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_APE_L), (mac_l+0x1));
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_APE_IP), 0x01010101);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_APE_UDP), 0xf000);

	//Do it again to be dead sure we didn't lose anything while PLL settles.
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_MAC_H), mac_h);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_MAC_L), mac_l);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_IP), bis->bi_ip_addr);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_APE_H), mac_h);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_CPU_APE_L), (mac_l+0x1));
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_APE_IP), 0x01010101);
	out_be32 (DANTE_APCP0_REG_BASE + (DANTE_APCP_APE_UDP), 0xf000);

	emac.isstarted = 1;

	printf("EMAC Initialization complete\n\r");
	return 0;
}

int eth_send(volatile void *ptr, int len)
{
	u32 intrstatus;
	u32 fifocount;
	u32 wordcount;
	u32 extrabytecount;
	u32 *wordbuffer = (u32 *) ptr;

	if (len > ENET_MAX_MTU)
		len = ENET_MAX_MTU;

	/*
	 * Check for overruns and underruns for the transmit status and length
	 * FIFOs and make sure the send packet FIFO is not deadlocked.
	 * Any of these conditions is bad enough that we do not want to
	 * continue. The upper layer software should reset the device to resolve
	 * the error.
	 */
	intrstatus = in_be32 ((emac.baseaddress) + DEMAC_INTR_ISR_OFFSET);
	if (intrstatus & (DEMAC_INTR_XMIT_SFIFO_OVER_MASK |
			DEMAC_INTR_XMIT_LFIFO_OVER_MASK)) {
		debug ("Transmitting overrun error\n");
		return 0;
	} else if (intrstatus & (DEMAC_INTR_XMIT_SFIFO_UNDER_MASK |
			DEMAC_INTR_XMIT_LFIFO_UNDER_MASK)) {
		debug ("Transmitting underrun error\n");
		return 0;
	}


	/*
	 * Before writing to the data FIFO, make sure the length FIFO is not
	 * full. The data FIFO might not be full yet even though the length FIFO
	 * is. This avoids an overrun condition on the length FIFO and keeps the
	 * FIFOs in sync.
	 *
	 * Clear the latched LFIFO_FULL bit so next time around the most
	 * current status is represented
	 */
	if (intrstatus & DEMAC_INTR_XMIT_LFIFO_FULL_MASK) {
		out_be32 ((emac.baseaddress) + DEMAC_INTR_ISR_OFFSET,
			intrstatus & DEMAC_INTR_XMIT_LFIFO_FULL_MASK);
		debug ("Fifo is full\n");
		return 0;
	}

	/* get the count of how many words may be inserted into the FIFO */
	fifocount = in_be32 (emac.baseaddress +
				DEMAC_TXDATA_CNT);
	fifocount = ((DEMAC_TXFIFO_DEPTH - fifocount) * 4);
	wordcount = len >> 2;
	extrabytecount = len & 0x3;

	if (fifocount < wordcount) {
		debug ("Sending packet is larger then size of FIFO\n");
		return 0;
	}

	for (fifocount = 0; fifocount < wordcount; fifocount++) {
		out_be32 (emac.sendfifo.databaseaddress, wordbuffer[fifocount]);
	}
	if (extrabytecount > 0) {
		u32 lastword = 0;
		u8 *extrabytesbuffer = (u8 *) (wordbuffer + wordcount);

		if (extrabytecount == 1) {
			lastword = extrabytesbuffer[0] << 24;
		} else if (extrabytecount == 2) {
			lastword = extrabytesbuffer[0] << 24 |
				extrabytesbuffer[1] << 16;
		} else if (extrabytecount == 3) {
			lastword = extrabytesbuffer[0] << 24 |
				extrabytesbuffer[1] << 16 |
				extrabytesbuffer[2] << 8;
		}
		out_be32 (emac.sendfifo.databaseaddress, lastword);
	}

	/*
	 * Set the MAC's transmit packet length register to tell it to transmit
	 */
	out_be32 (emac.baseaddress + DEMAC_TXFLEN, len);

	/*
	 * Loop on the MAC's status to wait for the transmit to complete.
	 * The transmit status is in the FIFO when the XMIT_DONE bit is set.
	 */
	do {
		intrstatus = in_be32 ((emac.baseaddress) +
						DEMAC_INTR_ISR_OFFSET);
	}
	while ((intrstatus & DEMAC_INTR_XMIT_DONE_MASK) == 0);

	if (intrstatus & (DEMAC_INTR_XMIT_SFIFO_OVER_MASK |
					DEMAC_INTR_XMIT_LFIFO_OVER_MASK)) {
		debug ("Transmitting overrun error\n");
		return 0;
	} else if (intrstatus & (DEMAC_INTR_XMIT_SFIFO_UNDER_MASK |
					DEMAC_INTR_XMIT_LFIFO_UNDER_MASK)) {
		debug ("Transmitting underrun error\n");
		return 0;
	}

	return 1;
}

int eth_rx(void)
{
	u32 pktlength;
	u32 intrstatus;
	u32 fifocount;
	u32 wordcount;
	u32 extrabytecount;
	u32 lastword;
	u8 *extrabytesbuffer;
	
	/*
	 * Get the interrupt status to know what happened (whether an error
	 * occurred and/or whether frames have been received successfully).
	 * When clearing the intr status register, clear only statuses that
	 * pertain to receive.
	 */
	intrstatus = in_be32 ((emac.baseaddress) + XIIF_V123B_IISR_OFFSET);
	/*
	 * Before reading from the length FIFO, make sure the length FIFO is not
	 * empty. We could cause an underrun error if we try to read from an
	 * empty FIFO.
	 */
	if ((intrstatus & DEMAC_INTR_RECV_LFIFO_EMPTY_MASK)) {
		/* debug ("Receiving FIFO is empty\n"); */
		return 0;
	}

	/*
	 * Determine, from the MAC, the length of the next packet available
	 * in the data FIFO (there should be a non-zero length here)
	 */
	pktlength = in_be32 (emac.baseaddress + DEMAC_RXFLEN);
	if (!pktlength) {
		puts("zero pkt len\n\r\0");
		return 0;
	}

	/*
	 * Write the RECV_DONE bit in the status register to clear it. This bit
	 * indicates the RPLR is non-empty, and we know it's set at this point.
	 * We clear it so that subsequent entry into this routine will reflect
	 * the current status. This is done because the non-empty bit is latched
	 * in the IPIF, which means it may indicate a non-empty condition even
	 * though there is something in the FIFO.
	 */
	out_be32 ((emac.baseaddress) + XIIF_V123B_IISR_OFFSET,
						DEMAC_INTR_RECV_DONE_MASK);
	
	fifocount = in_be32 (emac.baseaddress + DEMAC_RDFIFO_SR_OFFSET)
				& RDFIFO_OCC_MASK;

	wordcount = pktlength >> 2;
	extrabytecount = pktlength & 0x3;

	if ((fifocount) < wordcount) {
		puts ("Receiving FIFO is smaller than packet size.\n\r\0");
		return 0;
	}

	for (fifocount = 0; fifocount < wordcount; fifocount++) {
		etherrxbuff[fifocount] =
				in_be32 (emac.recvfifo.databaseaddress);
	}

	/*
	 * if there are extra bytes to handle, read the last word from the FIFO
	 * and insert the extra bytes into the buffer
	 */
	if (extrabytecount > 0) {
		extrabytesbuffer = (u8 *) (etherrxbuff + wordcount);

		lastword = in_be32 (emac.recvfifo.databaseaddress);

		/*
		 * one extra byte in the last word, put the byte into the next
		 * location of the buffer, bytes in a word of the FIFO are
		 * ordered from most significant byte to least
		 */
		if (extrabytecount == 1) {
			extrabytesbuffer[0] = (u8) (lastword >> 24);
		} else if (extrabytecount == 2) {
			extrabytesbuffer[0] = (u8) (lastword >> 24);
			extrabytesbuffer[1] = (u8) (lastword >> 16);
		} else if (extrabytecount == 3) {
			extrabytesbuffer[0] = (u8) (lastword >> 24);
			extrabytesbuffer[1] = (u8) (lastword >> 16);
			extrabytesbuffer[2] = (u8) (lastword >> 8);
		}
	}
	NetReceive((uchar *)etherrxbuff, pktlength);
	return 1;
}
